@@ -14,34 +14,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from io import BytesIO
 import os
+from io import BytesIO
 
-from ... import (
-    revision as _mod_revision,
-    tests,
-    trace,
-    )
-from ...diff import show_diff_trees
-from ...merge import Merger, Merge3Merger
-from ...transform import (
-    ROOT_PARENT,
-    resolve_conflicts,
-    )
-from ...tree import (
-    find_previous_path,
-    TreeChange,
-    )
-
-from breezy.tests.per_tree import TestCaseWithTree
 from breezy.tests.matchers import MatchesTreeChanges
+from breezy.tests.per_tree import TestCaseWithTree
 
-
-from ..features import (
-    SymlinkFeature,
-    UnicodeFilenameFeature,
-    )
-
+from ... import revision as _mod_revision
+from ... import tests, trace
+from ...diff import show_diff_trees
+from ...merge import Merge3Merger, Merger
+from ...transform import ROOT_PARENT, resolve_conflicts
+from ...tree import TreeChange, find_previous_path
+from ..features import SymlinkFeature, UnicodeFilenameFeature
 
 
 class TestTransformPreview(TestCaseWithTree):
@@ -627,7 +612,7 @@ class TestTransformPreview(TestCaseWithTree):
         preview.new_file('new-versioned-file', preview.root, [b'contents'],
                          b'new-versioned-id')
         tree = preview.get_preview_tree()
-        self.assertEquals({'existing-file'}, set(work_tree.extras()))
+        self.assertEqual({'existing-file'}, set(work_tree.extras()))
         preview.unversion_file(preview.trans_id_tree_path('removed-file'))
         self.assertEqual({'new-file', 'removed-file', 'existing-file'},
                          set(tree.extras()))
@@ -744,6 +729,6 @@ class TestTransformPreview(TestCaseWithTree):
         tt = tree.preview_transform()
         self.addCleanup(tt.finalize)
         foo_id = tt.new_directory('', ROOT_PARENT)
-        bar_id = tt.new_file(u'\u1234bar', foo_id, [b'contents'])
+        bar_id = tt.new_file('\u1234bar', foo_id, [b'contents'])
         limbo_path = tt._limbo_name(bar_id)
         self.assertEqual(limbo_path, limbo_path)
