@@ -21,17 +21,12 @@ import os
 
 from testtools import content
 
-from .. import (
-    plugins as _mod_plugins,
-    trace,
-    )
-from ..controldir import ControlDir
+from .. import plugins as _mod_plugins
+from .. import trace
 from ..bzr.smart import medium
+from ..controldir import ControlDir
 from ..transport import remote
-
-from . import (
-    TestCaseWithTransport,
-    )
+from . import TestCaseWithTransport
 
 old_format_modules = [
     'breezy.bzr.knitrepo',
@@ -68,7 +63,7 @@ class ImportTariffTestCase(TestCaseWithTransport):
         for name in ('BRZ_PLUGIN_PATH', 'BRZ_DISABLE_PLUGINS', 'BRZ_PLUGINS_AT'
                      ):
             self.preserved_env_vars[name] = os.environ.get(name)
-        super(ImportTariffTestCase, self).setUp()
+        super().setUp()
 
     def start_brz_subprocess_with_import_check(self, args, stderr_file=None):
         """Run a bzr process and capture the imports.
@@ -184,7 +179,6 @@ class TestImportTariffs(ImportTariffTestCase):
             'getpass',
             'kerberos',
             'merge3',
-            'shutil',
             'ssl',
             'socket',
             'smtplib',
@@ -237,10 +231,8 @@ class TestImportTariffs(ImportTariffTestCase):
             'getpass',
             'kerberos',
             'merge3',
-            'shutil',
             'smtplib',
             'ssl',
-            'tempfile',
             'tarfile',
             'termios',
             'tty',
@@ -307,7 +299,6 @@ class TestImportTariffs(ImportTariffTestCase):
                                       'merge3',
                                       'smtplib',
                                       'tarfile',
-                                      'tempfile',
                                       'termios',
                                       'tty',
                                       ] + old_format_modules)
